'.-------+----------+--------- [MultiExec] -------+--------------------:
'|                     MultiExec System v1.00 by TuS                   |
':-------+----------+----------+-----------------+---------------------:

Option Explicit 

'Import Configuration
Import ("MultiExec.Config")

'Make sure constants in MultiExec.Config are correctlly defined
If TypeName(RAIDENFTPD_PATH) <> "String" _ 
	Or TypeName(NUMBEROFLOGS) <> "Integer" _ 
	Or TypeName(FILELOG) <> "Integer" _
	Or TypeName(EVENTLOG) <> "Integer" Then
	WScript.echo "One or more required constants is not defined. Please see " _
	& "readme.txt for installation"
	WScript.quit
End If

'*********************************************************************************
'*********************************************************************************
'       DO NOT CHANGE THESE CONSTANTS UNLESS YOU KNOW WHAT YOU ARE DOING
'*********************************************************************************
Const APPNAME       = "TuS MultiExec"
Const APPVERMAJOR   = "1"
Const APPVERMINOR   = "0"
Const FORREADING    = 1
Const FORWRITING    = 2
Const FORAPPENDING  = 8
Const SUCCESS       = 0
Const ERRORL        = 1
Const WARNING       = 2
Const INFORMATION   = 4
Const ABORT         = "Script aborted with error(s). To determine cause, see logged information"
'Script Root folder
Dim SCRIPTPATH : SCRIPTPATH = Left(WScript.ScriptFullName, InStrRev(WScript.ScriptFullName,"\")-1)
'Script Base Params
'Idenitify GCID/Server/eventType parameters position in the MultiExec Script Argument list
Const CMDGCID 	= 1
Const CMDSERVER = 2
Const CMDTYPE	= 3
'*********************************************************************************
'012345678901234567890123456789012345678901234567890123456789012345678901234567891

'#################################################################################
'	Main Entry Point
'#################################################################################
Dim iCnt, iCnt2, ret, sFile, response
Dim Command, tmpGCiD, GCiD, sArgs
Dim oArgs : Set oArgs = WScript.Arguments

Dim currentSub : currentSub = "Main Code"
Dim tStartTime,tEndTime

Const DEBUG_CLASS_EVENTS = true
Const FIRE_CLASS_EVENTS  = true

tStartTime = Timer()

'#-------------------------------------------------------------------------------#
'Load the framwork
'#-------------------------------------------------------------------------------#

'Init the Logger
Dim UserLog : Set UserLog = New UserLogClass
UserLog.LogEvent SUCCESS, "*** " & APPNAME & " " & APPVERMAJOR & "." & APPVERMINOR & " ***"

'Init the Helper
Dim Helper : Set Helper = New HelperClass

'Start the logger
UserLog.Start
'#-------------------------------------------------------------------------------#
UserLog.LogEvent SUCCESS, "'#--------------------------------------------------------------------------------#"
UserLog.LogEvent SUCCESS, "[Main Code] Script Started"

'Debug Main Parameters
UserLog.LogEvent SUCCESS, "[" & currentSub & "] " &  "Processing with "& oArgs.Count & " Input Arguments:"
For iCnt = 0 To oArgs.Count -1 
	sArgs = sArgs & "|" & oArgs(iCnt)
'	UserLog.LogEvent INFORMATION, "[" & currentSub & "]   - Argument#" & iCnt+1 & ", Value=" & oArgs(iCnt)
Next	
UserLog.LogEvent SUCCESS, "[" & currentSub & "]   - Arguments:" & sArgs
'Check if any commands are defined
If (NUMCOMMANDS > 0)  Then
	For iCnt = 1 To NUMCOMMANDS
		'Check if we have enough Args
		If  (oArgs.Count >= Eval("CMD" & iCnt & "ARGS")) Then
			'Build the command string
			If (InStr(Eval("CMD" & iCnt),Chr(32))) Then		
				Command = Chr(34) & Eval("CMD" & iCnt) & Chr(34)
			Else 
				Command = Eval("CMD" & iCnt)
			End If 
			
			'Build the command Argument List
			For iCnt2 = 1 To Eval("CMD" & iCnt & "ARGS")
				If ((TypeName(Eval("CMD" & iCnt & "ARG" & iCnt2 )) = "Integer") And (Eval("CMD" & iCnt & "ARG" & iCnt2) <= oArgs.Count)) Then
					Command = AddArgument(Command,oArgs(Eval("CMD" & iCnt & "ARG" & iCnt2)-1),Eval("CMD" & iCnt & "ARG" & iCnt2 & "Quotes"))
				Else 
					Command = AddArgument(Command,Eval("CMD" & iCnt & "ARG" & iCnt2),Eval("CMD" & iCnt & "ARG" & iCnt2 & "Quotes"))
				End If 
			Next
	
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] # Command "& iCnt & ": [" & Command & "]"
			
			'Run the command
			ret = Helper.ShellRun(Command,0,Eval("CMD" & iCnt & "ASK"))
			
			'Check if the command is providing an answer
			If (Eval("CMD" & iCnt & "ASK")) Then
				UserLog.LogEvent INFORMATION, "[" & currentSub & "] # Command " & iCnt & ": Checking ASK Result"
				'set the file to read
				sFile = RAIDENFTPD_PATH & "\OutPut" & "\ask-" & oArgs(CMDTYPE-1) & "." & oArgs(CMDGCID-1)
				'check if any response
				response = Helper.ReadFileLine(sFile,1)
				If (response <> vbEmpty) Then
					'check if response is True or False
					UserLog.LogEvent SUCCESS, "[" & currentSub & "] # Command " & iCnt & ": Checking ASK Result"
					If (CBool(response) = False) Then
						UserLog.LogEvent SUCCESS, "[" & currentSub & "] # Command " & iCnt & ": ASK Response is "& response & ". Abort command Processing"
						Exit For
					Else
						UserLog.LogEvent SUCCESS, "[" & currentSub & "] # Command " & iCnt & ": ASK Response is "& response & ". Continue with following commands"
					End If 
				Else
					UserLog.LogEvent ERRORL, "[" & currentSub & "] # Command " & iCnt & ": unable to evaluate ASK Response. Abort command Processing"
					Exit For
				End if
			Else
				UserLog.LogEvent INFORMATION, "[" & currentSub & "] # Command " & iCnt & ": is not an ASK. Continue with Next command"
				
			End If 
		Else
			UserLog.LogEvent ERRORL, "[" & currentSub & "] # Command " & iCnt & ": Configuration ERROR : Not enough arguments"
		End If 
	Next
Else
	UserLog.LogEvent ERRORL, "[Main Code] No commands to execute or parameter Problem. Check your configuration."
End if

UserLog.LogEvent SUCCESS, "[Main Code] Script Completed"

Function AddArgument(sCmd,sArg,bQuotes)
	
	'Default 
	AddArgument = sCmd & " " & sArg
	
	If TypeName(bQuotes) = "Boolean" Then
		If (bQuotes) Then
			AddArgument = sCmd & " " & Chr(34) & sArg & Chr(34)
		End If
	End If
End function


'#################################################################################
'	Main Exit Point
'#################################################################################


'######################################################################################
' Class Wrapper for Logging
'######################################################################################
Class UserLogClass
	
	Private bLoaded, bLogActive
	Private	sLogRoot,sLog
	Private oShell,oFso,oRegEx,oLog,oFldr,oFile
	Private iLogToKeep,iArr,iCnt,n,m,temp
	Private	aLog,aDate
	Private cFiles
	
	Private Sub Class_Initialize
		'#--------------------------------------------------------------------------------#
		' Purpose : Class Initialisation
		'#--------------------------------------------------------------------------------#	
		iLogToKeep	= NUMBEROFLOGS
		bLoaded		= False
		bLogActive	= False
		sLogRoot 	= SCRIPTPATH & "\Log"
		Set oShell 	= WScript.CreateObject("Wscript.Shell")
		Set oFso	= CreateObject("Scripting.FileSystemObject")
	End Sub
	
	Private Sub Class_Terminate
		'#--------------------------------------------------------------------------------#
		' Purpose : Class Terminate
		'#--------------------------------------------------------------------------------#	
		Dim currentSub : currentSub = "Class UserLog:Class_Terminate"
		
		'Clean up the logs
	DelLog
		
		Set oFso	= Nothing
		Set oShell	= Nothing
	End Sub
	
	Public Property Get LogPath()
	'#--------------------------------------------------------------------------------#
	' Purpose	: This function sets the file path
	'#--------------------------------------------------------------------------------#
	LogPath = sLogRoot
	End Property
	
	Public Property Let LogPath(sPath)
	'#--------------------------------------------------------------------------------#
	' Purpose	: This function sets the file path
	'#--------------------------------------------------------------------------------#
	If sPath <> vbNullString Then
		sLogRoot = sPath
	End If 
	End Property
	
	Public Property Get LogActive()
	'#--------------------------------------------------------------------------------#
	' Purpose	: This function sets the file path
	'#--------------------------------------------------------------------------------#
	LogActive = bLogActive
	End Property
	
	Public Property Get LogsToKeep()
	'#--------------------------------------------------------------------------------#
	' Purpose	: This function sets the file path
	'#--------------------------------------------------------------------------------#
	LogsToKeep = iLogToKeep
	End Property
	
	Public Property Let LogsToKeep(iValue)
	'#--------------------------------------------------------------------------------#
	' Purpose	: This function sets the file path
	'#--------------------------------------------------------------------------------#
	If (VarType(iValue) = vbInteger) Then
		iLogToKeep = iValue
	End If 
	End Property
	
	Public Property Get LogFile()
	'#--------------------------------------------------------------------------------#
	' Purpose	: This function sets the file path
	'#--------------------------------------------------------------------------------#
	LogFile = sLog
	End Property
	
	Public Function Start()
		'**********************************************************************************
		' Purpose : Initialize the Log interface, check log path
		'           Easily extend logging into other formats, Ex : Syslog, mail, xml etc.
		' Input   : none
		' Return  : bool	: True|False
		'**********************************************************************************
		Dim currentSub : currentSub = "Class UserLog:Initialize"
		
		If (Helper.CreateFolder(sLogRoot) = -1) Then
			LogEvent ERRORL, ABORT
			WScript.quit
		End If
		
		sLog = sLogRoot & "\" & Helper.GetDayTimeDate() & ".log"
		If (oFso.FileExists(sLog)) Then
			LogEvent INFORMATION, "[" & currentSub & "] The logfile: " & sLog & " allready exists. Appending random number to name of logfile"
			sLog = sLogRoot & "\" & Helper.GetDayTimeDate() & "_" & Helper.Random(9999) & ".log"
		End If
		
		LogEvent INFORMATION, "[" & currentSub & "] Logfile = " & sLog
		
		bLogActive = True
	End Function
	
	Public Sub LogEvent(eventType, sMessage)
		'**********************************************************************************
		' Purpose : Log interface, write various type of events to different formats, 
		'			NT Event, File etc.
		'           Easily extend logging into other formats, Ex : Syslog, mail, xml etc.
		' Input   : i : Event type, Integer 0 = None 1 = Success/Error 2 = Full
		'           sMessage : Log message, String
		'**********************************************************************************
		'Eventlog output
		If EVENTLOG = 1 Or EVENTLOG = 2 Then
			Set oRegEx = New RegExp
			'oRegEx.Pattern = "\[.*\] "
			'Changed to be able to print message file output when Message contains Brackets]
			oRegEx.Pattern = "^(\[[^*\]]*\])+"
			sMessage = oRegEx.Replace (sMessage, "")
			doEventLog eventType, Trim(sMessage)
			Set oRegEx = Nothing
		End If
		
		'Debug EventLog output
		If EVENTLOG = 3 Then
			doEventLog eventType, sMessage
		End If
		
		'Logfile output
		If FILELOG = 1 Or FILELOG = 2 Then
			Set oRegEx = New RegExp
			'oRegEx.Pattern = "\[.*\] "
			'Changed to be able to print message file output when Message contains Brackets]
			oRegEx.Pattern = "^(\[[^*\]]*\])+"
			sMessage = oRegEx.Replace (sMessage, "")
			doFileLog eventType, Trim(sMessage)
			Set oRegEx = Nothing
		End If
		
		'Debug logfile output
		If FILELOG = 3 Then
			doFileLog eventType, sMessage
		End If	
		
		'Debug screen output
		If FILELOG = 3 Or EVENTLOG = 3 Then
			WScript.echo sMessage
		End If
	End Sub
	
	Private Sub doFileLog(eventType, sMessage)
		'**********************************************************************************
		' Purpose : Write Event to logfile. Build an array of events until path logfile 
		'			is determined.
		'           If logfile can not be written. Events are redirected to eventlog.
		'           EventLog Level 0 = None 1 = Success/Error 2 = Full
		' Input   : eventType : Event type, Integer
		'           0 SUCCESS	1 ERROR (Const ERRORL)	2 WARNING	4 INFORMATION
		'           s : Log message, String
		' Use     : bLog Boolean True|False when actually start logging to file
		'**********************************************************************************
		On Error Resume Next
		
		Dim currentSub : currentSub = "Class UserLog:FileLog"
		
		'WScript.Echo eventType & " : " & sMessage
		'Make sure errorlevel 0
		Err.clear
		
		'Log to file if bLog flag is true
		If bLogActive = True Then
			
			Set oLog = oFso.OpenTextFile(sLog, FORAPPENDING, True)
			
			If Err.number <> 0 Then
				Helper.EchoErr Err, currentSub
				'Clear error
				Err.clear
				'Check if log folder exists, try to create
				If Helper.CreateFolder(Left(sLog, InStrRev(sLog, "\"))) <> -1 Then
					Set oLog = oFso.OpenTextFile(sLog, FORAPPENDING, True)
					
				Else
					
					'Abort filelog operation if can't be written, switch to EventLog	
					EVENTLOG = 2
					FILELOG = 0
					Err.Raise = iErr
					LogEvent ERRORL, "[" & currentSub & "] Error occured while writing to logfile: " & sLog & ". The error was: " & Err.description
					LogEvent WARNING, "[" & currentSub & "] Events will permanently be redirected to eventlog. All events occured prior to this warning will now be written."
					
					'Write events from array to eventlog
					If IsArray(aLog) And aLog(0) <> vbNullString Then
						For iArr = 0 To UBound(aLog)
							LogEvent CInt(Left(aLog(iArr), 1)), Right(aLog(iArr), Len(aLog(iArr)) - 1)
						Next
						
						ReDim aLog(0)	
					End If
					
					Exit Sub
				End If
			End If
			
			'Write events from array to logfile
			If IsArray(aLog) And aLog(0) <> vbNullString Then
				For iArr = 0 To UBound(aLog)
					oLog.WriteLine Right(aLog(iArr), Len(aLog(iArr)) - 1)
				Next
				
				ReDim aLog(0)
			End If
			
			Select Case FILELOG
				Case 1
				Select Case eventType
					Case SUCCESS
					oLog.WriteLine Now & " SUCCESS " & sMessage
					Case ERRORL
					oLog.WriteLine Now & " ERROR   " & sMessage
					Case Else
					'Exit Sub 
				End Select
				Case 2
				Select Case eventType
					Case SUCCESS
					oLog.WriteLine Now & " SUCCESS     " & sMessage
					Case ERRORL
					oLog.WriteLine Now & " ERROR       " & sMessage
					Case WARNING
					oLog.WriteLine Now & " WARNING     " & sMessage
					Case INFORMATION
					oLog.WriteLine Now & " INFORMATION " & sMessage
					Case Else
					'Exit Sub
				End Select
				Case 3
				Select Case eventType
					Case SUCCESS
					oLog.WriteLine Now & " SUCCESS     " & sMessage
					Case ERRORL
					oLog.WriteLine Now & " ERROR       " & sMessage
					Case WARNING
					oLog.WriteLine Now & " WARNING     " & sMessage
					Case INFORMATION
					oLog.WriteLine Now & " INFORMATION " & sMessage
					Case Else
					'Exit Sub
				End Select
				Case Else
				Exit Sub
			End Select
			
			oLog.close
			'Set oLog = Nothing
		Else
			
			'Log to array while path to logfile determined
			If IsArray(aLog) Then
				iArr = UBound(aLog) + 1
				ReDim Preserve aLog(iArr)
			Else
				ReDim aLog(0)
				iArr = UBound(aLog)
			End If
			
			Select Case FILELOG
				Case 1
				Select Case eventType
					Case SUCCESS
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " SUCCESS " & sMessage
					Case ERRORL
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " ERROR   " & sMessage
					Case Else
					ReDim Preserve aLog(iArr-1)
					Exit Sub
				End Select
				Case 2
				Select Case eventType
					Case SUCCESS
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " SUCCESS     " & sMessage
					Case ERRORL
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " ERROR       " & sMessage
					Case WARNING
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " WARNING     " & sMessage
					Case INFORMATION
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " INFORMATION " & sMessage
					Case Else
					Exit Sub
				End Select
				Case 3
				Select Case eventType
					Case SUCCESS
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " SUCCESS     " & sMessage
					Case ERRORL
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " ERROR       " & sMessage
					Case WARNING
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " WARNING     " & sMessage
					Case INFORMATION
					ReDim Preserve aLog(iArr)
					aLog(iArr) = eventType & Now & " INFORMATION " & sMessage
					Case Else
					Exit Sub
				End Select
				Case Else
				Exit Sub
			End Select
		End If
	End Sub
	
	Private Sub doEventLog(eventType, sMessage)
		'********************************************************************************
		' Purpose : Write Event to Eventlog.
		'           EventLog Level 0 = None 1 = Success/Error 2 = Full
		' Input   : eventType : Event type, Integer
		'           0 SUCCESS 1 ERROR (Const ERRORL) 2 WARNING 4 INFORMATION
		'           sMessage : Log message, String
		'********************************************************************************
		Select Case EVENTLOG
			Case 1
			Select Case eventType
				Case SUCCESS
				oShell.LogEvent eventType, sMessage
				Case ERRORL
				oShell.LogEvent eventType, sMessage
				Case Else
				Exit Sub
			End Select
			Case 2
			Select Case eventType
				Case SUCCESS
				oShell.LogEvent eventType, sMessage
				Case ERRORL
				oShell.LogEvent eventType, sMessage
				Case WARNING
				oShell.LogEvent eventType, sMessage
				Case INFORMATION
				oShell.LogEvent eventType, sMessage
				Case Else
				Exit Sub
			End Select
			Case 3
			Select Case eventType
				Case SUCCESS
				oShell.LogEvent eventType, sMessage
				Case ERRORL
				oShell.LogEvent eventType, sMessage
				Case WARNING
				oShell.LogEvent eventType, sMessage
				Case INFORMATION
				oShell.LogEvent eventType, sMessage
				Case Else
				Exit Sub
			End Select
			Case Else
			Exit Sub
		End Select
	End Sub
	
	Private Sub DelLog()
		'**********************************************************************************
		' Purpose : Delete obsolete logs
		' Input   : None
		'**********************************************************************************
		Dim currentSub : currentSub = "Class UserLog:DelLog"
		Dim sProcessTime,tEndTime
		
		UserLog.LogEvent SUCCESS, "'#--------------------------------------------------------------------------------#"
		UserLog.LogEvent SUCCESS, "[" & currentSub & "] Log CleanUP Started"
		
		If iLogToKeep <> 0 Then
			LogEvent INFORMATION, "[" & currentSub & "] Call KeepOnlyFiles " & sLogRoot & " " & iLogToKeep
			KeepOnlyFiles sLogRoot, iLogToKeep
		End If
		
		UserLog.LogEvent SUCCESS, "[" & currentSub & "] Log CleanUP Complete"
		UserLog.LogEvent SUCCESS, "'#--------------------------------------------------------------------------------#"
		
		tEndTime = Timer()
		sProcessTime = ((tEndTime - tStartTime)*1000)
		UserLog.LogEvent SUCCESS, "[" & currentSub & "] " & APPNAME & " " & APPVERMAJOR &  "." & APPVERMINOR & " Completed in " & sProcessTime & " milliseconds"
		
	End Sub
	
	Private Sub KeepOnlyFiles(sPath, iCount)
		'**********************************************************************************
		' Purpose : Delete obsolete files based upon date.
		'           n newest files are kept.
		' Input   : String (s) Path to root of files
		'           Integer : iCount : Number of folders to keep
		'**********************************************************************************
		'On Error Resume Next
		On Error Goto 0
		Err.Clear
		
		Dim cFiles
		Dim currentSub : currentSub = "Class UserLog:KeepOnlyFiles"
		
		Set oFldr = oFso.GetFolder(sPath)
		Helper.EchoErr Err, currentSub
		
		If Err.number <> 0 Then
			LogEvent ERRORL, "[" & currentSub & "] An Error occured connecting to: " & sPath & " old files must be deleted manually."
			Exit Sub
		Else
			LogEvent INFORMATION, "[" & currentSub & "] Connected to: " & sPath
		End If
		
		Set cFiles = oFldr.Files
		Helper.EchoErr Err, currentSub
		
		If Err.number <> 0 Then
			LogEvent ERRORL, "[" & currentSub & "] An Error occured while listing files in folder: " & sPath & ". Old files must be deleted manually."
			Exit Sub
		Else
			LogEvent INFORMATION, "[" & currentSub & "] Succsessfully listed files in folder: " & sPath
		End If
		
		'Check if number of files greater than i
		If cFiles.Count <= iCount Then
			LogEvent INFORMATION, "[" & currentSub & "] Number of file(s) in root is: " & cFiles.Count & ". Number is less or equal to: " & iCount & " wich is the number of file(s) to keep."
			Exit Sub
		End If
		
		LogEvent INFORMATION, "[" & currentSub & "] Number of files in root is: " & cFiles.Count & ". Number is greater: " & iCount & " wich is the number of file(s) to keep."
		
		'ReDim array to number of folders
		ReDim aDate(cFiles.Count -1)
		
		LogEvent INFORMATION, "[" & currentSub & "] Creating array with fixed size: " & cFiles.Count -1 & ". Adding date of folders to array"
		
		iCnt = 0
		
		'Add date creation of all folders to array
		For Each oFile In cFiles
			LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & oFile.DateCreated & " (Path: " & oFile.Path & ")"
			aDate(iCnt) = oFile.DateCreated
			iCnt = iCnt + 1
		Next
		
		'Sorting array by date, ascent sort order (The famous bubble sort)
		LogEvent INFORMATION, "[" & currentSub & "] Sorting dates in array, ascent sort order"
		For n = 0 To UBound(aDate) -1 
			For m = n+1 To UBound(aDate) 
				If aDate(m) < aDate(n) Then
					temp = aDate(m)
					aDate(m) = aDate(n)
					aDate(n) = temp
				End If
			Next
		Next
		
		'Just logging
		LogEvent INFORMATION, "[" & currentSub & "] Array sorted, result in ascent order: "
		For iCnt = 0 To UBound(aDate)
			LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & aDate(iCnt)
		Next
		
		'Cut array, keep (i) rows with date of folders wich is going to be deleted
		ReDim Preserve aDate(UBound(aDate) - iCount)
		LogEvent INFORMATION, "[" & currentSub & "] Array aDate cut, remaing rows are: "
		
		'Even more logging (those debuggers ;))
		For iCnt = 0 To UBound(aDate)
			LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & aDate(iCnt)
		Next
		
		LogEvent INFORMATION, "[" & currentSub & "] Start deleting file(s)..."
		For Each oFile In cFiles
			For iCnt = 0 To UBound(aDate)
				If oFile.DateCreated = aDate(iCnt) Then
					LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & aDate(iCnt) & " matches DateCreated on file: " & oFile.Path & " (Date=" & oFile.DateCreated & ")"
					Helper.DeleteFile oFile.Path
					Exit For
				End If
			Next
		Next
		
		Helper.EchoErr Err, currentSub
		
		If Err.Number = 0 Then
			LogEvent SUCCESS, "[" & currentSub & "] Deleted " & UBound(aDate)+1 & " obsolete log files" 
		End If
		
	End Sub
End Class
'######################################################################################
' Class Wrapper for Helper. Functions
'######################################################################################
Class HelperClass
	
	Private oShell,oExec,oFso,oFile,oStdOut,Element,cmdOutput
	Private iSize,iInt,iCnt,iWordBreak,iCurrentLenth
	Private sScriptDir, sFilePath, sStreamData, sFolder, sUnc, sResult, sTempFile
	Private aResult
	Private bFound
	Private objRegExp, strPattern
	'GetDayTimeDate
	Public intSeconds, intMilliseconds
	'KeepOnly
	Private cCollection, oObject, sType, aDate, oFldr, n, m, temp, iCnt2, aDeleted()
	'GetFolderContent
	Private folder, items, item, FolderExists2, sType2, arrList(),i,sName
	'#---------------------------------------------------------------------------#
	Private Sub Class_Initialize
		Set oFso		= CreateObject("Scripting.FileSystemObject")
		Set oShell		= CreateObject("Wscript.Shell")
	End Sub
	
	Private Sub Class_Terminate
		Set oFso		= Nothing
		Set oShell		= Nothing
	End Sub
	'#---------------------------------------------------------------------------#
	
	Function EchoErr(myError, sProcedure)
		'**********************************************************************************
		' Purpose : Verify and Post Err message
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:EchoErr"
		
		If Err.Number <> 0 Then
			'WScript.echo "["& sProcedure & "] ERROR: Number:" & Err.Number & " : " & Err.Description
			UserLog.LogEvent ERRORL, "["& sProcedure & "] ERROR: Number:" &Err.Number & " : " & Err.Description
		End If
	End Function
	
	Sub import (sFileName)
		'**********************************************************************************
		' Purpose : Import an external script file 
		' Inputs  : Srting : sFileName : the file name
		' Output  : None
		'**********************************************************************************
		sScriptDir = oFso.GetParentFolderName(WScript.ScriptFullName)   
		sFilePath = oFso.BuildPath(sScriptDir,sFileName)   
		Const ForReading = 1   
		
		Set oFile = oFso.OpenTextFile(sFilePath,ForReading)   
		sStreamData = oFile.ReadAll()   
		ExecuteGlobal sStreamData   
	End Sub

	Public Function GetTempFilename()
		'**********************************************************************************
		' Purpose : Generate e temporary file, 
		' Inputs : None
		' Output : StringArray  : ShellRun : the actual output if any
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:GetTempFilename"
		
		GetTempFilename = oShell.ExpandEnvironmentStrings("%temp%") & oFso.GetTempName
	End Function	

	Public Function ShellRun(sCmd, iWinStyle, bWait)
		'**********************************************************************************
		' Purpose : Execute a command, It will then execute the command in a shell, 
		'           and capture the output into a file. That file is then read in and its 
		'           contents are returned as an Array() in the value the function returns.
		' Inputs : String : sCmd : The command to execute
		'		   Integer: iWinStyle : the window style (check VBS help for values)
		'		   Bool   : bWait : Wait for execution complete and return the output
		' Output : Array  : ShellRun : the actual output if any
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:ShellRun"
				
		sTempFile = GetTempFilename()
		On Error Resume Next 
		
		Dim sCmdOnly
		Dim sCmdPath
		Dim sCmdDrive
		Dim COMSPEC : COMSPEC = "%comspec% /c "
		
		UserLog.LogEvent Information, "[" & currentSub & "] Command is : ["  & sCmd  & "]"
		
		'Build the %comspec% to set drive and work directory
		If InStr(sCmd,Chr(34)) = 1 Then 'command is quoted
			sCmdOnly = Left(sCmd, InStr(2,sCmd,Chr(34)))
			sCmdOnly = Replace(sCmdOnly,Chr(34),vbNullString)
		Else
			sCmdOnly = Left(sCmd, InStr(2,sCmd,Chr(32)))
			sCmdOnly = Replace(sCmdOnly,Chr(32),vbNullString)
		End If 	
		
		If InStr(sCmdOnly,":") = 2 Then 
			'extract drive info
			sCmdDrive = Left(sCmdOnly, InStr(sCmdOnly,":"))
			COMSPEC = COMSPEC & sCmdDrive & " " & chr(38) & " "
			'extract Path info
			sCmdpath = Left(sCmdOnly, InStrRev(sCmdOnly,"\")-1)
			If InStr(sCmdPath,Chr(32)) > 0 Then
				sCmdPath = Chr(34) & sCmdPath & Chr(34)
			End if
			COMSPEC = COMSPEC & "cd " & sCmdPath & " " & chr(38) & " "
		End If 
				
		If bWait Then
			oShell.Run COMSPEC &  sCmd  & " > " & sTempFile, iWinStyle, true
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] Command Execute (Wait): [" & COMSPEC & sCmd  & " > " & sTempFile & "]"
			If Helper.FileExists(sTempFile,False) Then
				cmdOutput = Split(oFso.OpenTextFile(sTempFile,1).ReadAll,vbCRLF)
				If Err.Number = 62 Then
					UserLog.LogEvent WARNING, "[" & currentSub & "] No Output result from Command Execute"
					'Clear error
					Err.Clear
				Else
					'Just Debug
					UserLog.LogEvent SUCCESS, "[" & currentSub & "] Command has Output."
					For iCnt = 0 To UBound(cmdOutput)
						UserLog.LogEvent INFORMATION, "[" & currentSub & "] -" & cmdOutput(iCnt)
					Next
				End If
	
				oFso.DeleteFile sTempFile, true
			Else
				UserLog.LogEvent WARNING, "[" & currentSub & "] No Output File Command Execute"
			End If  
			ShellRun = cmdOutput
		Else 
			oShell.Run COMSPEC & sCmd, iWinStyle, false
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] Command Execute (No Wait): " & sCmd
			ShellRun = Array()
		End If 
			
		If Err.Number <> 0 Then
			Helper.EchoErr Err, currentSub
			'Clear error
			Err.clear
		End If
	End Function

	Public Function ShellExec(sCmd, bWait)
		'**********************************************************************************
		' Purpose : Execute a command
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:ShellExec"
				
		If (Not IsNothing(oShell)) then
			Set oExec = oShell.exec ("%comspec% /c " & Chr(34) & sCmd & Chr(34))
		
			If (bWait) then
				'Looping allowing it to finish
				Do Until oExec.Status = "1"
					wscript.sleep 100
				Loop
				
				'Create sdtout object for redirect of output from sCmd
		  		Set oStdOut = oExec.StdOut
				
				'Grabbing output from sCmd
				UserLog.LogEvent INFORMATION, "[" & currentSub & "] Redirecting output from : " & sCmd
				While Not oStdOut.AtEndOfStream
	 				UserLog.LogEvent INFORMATION, "[" & currentSub & "] Output : " & oStdOut.ReadLine
				Wend
			End If 

			If oExec.ExitCode <> 0 Then
			     	UserLog.LogEvent WARNING, "[" & currentSub & "] Warning: Non-zero exit code"
			End If
			
			'Return the exit code
			ShellExec = oExec.ExitCode

			On Error Goto 0
			If Err.Number <> 0 Then
					Helper.EchoErr Err, currentSub
					'Clear error
					Err.clear
			End If
		End If 
	End Function


	Public Function Validate(sStringToTest,aArray)
		'**********************************************************************************
				' Purpose : Check if the element is part of an array
		'**********************************************************************************
		bFound = False
		
		If (IsArray(aArray)) Then
			For Each Element In aArray				
				If LCase(sStringToTest) = LCase(Element) Then
					bFound = True
					Exit For
				End If
			Next 
		End If
		Validate = bFound
	End Function 
	

	Public Function FileMatchesPattern(strFileName, strWildCard) 
		'**********************************************************************************
		' Function: FileMatchesPattern(strFileName, strWildCard) 
		'**********************************************************************************
		' Function to test if a filename matches the wildcard characters passed. 
		' Converts a wildard (*.vbs) to a valid regular expression so that the file name can be tested  
		' for a match using a regular expression object.  This negates the need to perform a lot of  
		' string parsing and comparison. 
		'         
		' Params: 
		'       strFileName             String, Holding the name of the file to test (must not include the path) 
		'       strWildCard             String, Holding the wildcard string used to compare the file with. (eg. "*.vbs") 
		' Returns:  
		'       True if the filename matches the wildcard, otherwise False. 
		'**********************************************************************************	  
        Set objRegExp = CreateObject("VBScript.RegExp") 
         
        ' Update the wildcard string to define a valid regular expression 
        strPattern = Replace(strWildCard, ".", "\.") 
        strPattern = Replace(strPattern, "*", ".*") 
        strPattern = "^" & strPattern & "$" 
         
        With objRegExp 
                .Pattern = strPattern 
                .IgnoreCase = True 
                .Global = True 
        End With 
         
        FileMatchesPattern = objRegExp.Test(strFileName) 
        'WScript.Echo "File: [" & strFileName & "] matches Pattern: [" & strPattern & "] = " &  FileMatchesPattern 
        Set objRegExp = Nothing 
	End Function 

	Public Function GetSize(varArray)
		'**********************************************************************************
		' Purpose : Check the array size
		'**********************************************************************************
		iSize = 0
		
		On Error Resume Next
		iSize = UBound(varArray)
		If Err.Number Then
			If Err.Number = 9 Then
				iSize = 0
			End If
		Else
			iSize = UBound(varArray) + 1
		End If
		GetSize = iSize
	End Function 
	
	Public Function IsNothing (Obj)
		'**********************************************************************************
		' Purpose : Check if object is Nothing
		'**********************************************************************************
		If TypeName(Obj) = "Nothing" Then
			IsNothing = True
		Else
			IsNothing = False
		End If
	End Function
	
	Function FileExists(sFilePath, bRequired)
		'**********************************************************************************
		' Purpose : Verify presense of a file, abort the script if the file is not present
		'		    and it's required.
		' Input   : string	: sFilePath 		: Path to file
		'           bool	: bRequired	: True|False
		' Return  : bool	: True|False
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:FileExists"
		
		If (oFso.FileExists(sFilePath)) Then
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] The file: " & sFilePath & " is present."
			FileExists = True
		Else
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] The file: " & sFilePath & " does not exist."
			FileExists = False
		End If
		
		If bRequired And FileExists = False Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] The file: " & sFilePath & " is required for futher processing of this script."
		End If
	End Function
	
	Function FolderExists(sFolderPath, bRequired)
		'**********************************************************************************
		' Purpose : Verify presense of a folder, abort the script if the folder is not present
		'		    and it's required.
		' Input   : string	: sFFolderPath 		: Path to folder
		'           bool	: bRequired	: True|False
		' Return  : bool	: True|False
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:FolderExistsExists"
		
		If (oFso.FolderExists(sFolderPath)) Then
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] The Folder: " & sFolderPath & " is present."
			FolderExists = True
		Else
			UserLog.LogEvent ERRORL, "[" & currentSub & "] The Folder: " & sFolderPath & " does not exist."
			FolderExists = False
		End If
		
		If bRequired And FolderExists = False Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] The folder: " & sFolderPath & " is required for futher processing of this script."
			CmdProcessor.TerminateInSysErr
		End If
	End Function
	
	Function GetFolderContent(sFolderPath, bType, sMask)
		'**********************************************************************************
		' Purpose : Gets the content of a folder, abort the script if the folder is not present
		'		    and it's required.
		' Input   : string	: sFolderPath 		: Path to folder
		'			bool	: bType False:Files | True:Folder
		' Return  : Array	: Array of file/folder Names
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:GetFolderContent"
		
		If (oFso.FolderExists(sFolderPath)) Then
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] The Folder: " & sFolderPath & " is present."
			
			Set folder = oFso.GetFolder(sFolderPath)
			
			Select Case bType
				Case True
					sType2 = "Folder"
					Set items = folder.SubFolders
				Case False
					sType2 = "File"
					Set items = folder.Files
			End Select
			
			If items.count > 0 then
				i=0
				For each item In items
					If Helper.FileMatchesPattern (item.Name, sMask) then
						UserLog.LogEvent INFORMATION, "[" & currentSub & "] Listing " & sType2 & ": " & item.Name
						ReDim Preserve arrList(i)
						arrList(i) = item.Name
						i = i + 1
					End if
				Next
			End If 
			GetFolderContent = arrList

		Else
			UserLog.LogEvent ERRORL, "[" & currentSub & "] The Folder: " & sFolderPath & " does not exist."
			Set GetFolderContent = Array()
		End If
	End Function
		
	Function CreateFolder(sPath)
		'**********************************************************************************
		' Purpose : Create a folder
		' Input   : string	: sPath	: Path to folder
		' Return  : integer	:  -1 : folder creation error
		'						0 : folder allready exists
		'						1 : folder successfully created
		'**********************************************************************************
		sFolder = vbnullString
		
		Dim currentSub : currentSub = "Class Helper:CreateFolder"
		
		On Error Resume Next
		
		'Add missing \ if required
		If Right(sPath, 1) <> "\" Then
			sPath = sPath & "\"
		End If
		
		'Check if path is UNC replace base path \\server.fqdn\something with "\"
		If Left(sPath, 2) = "\\" Then
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] UNC Path: " & sPath
			sUNC = sPath
			Set oRegEx = New RegExp
			oRegEx.Pattern = "\\\\[A-Za-z0-9.]+\\\w+\$?\\"
			sResult = oRegEx.Replace (sPath, "")
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] UNC stripped, result: " & sResult
			sUnc = Left(sPath, Len(sPath) - Len(sResult))
			sFolder = sUnc
		End If
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Start creating directory structure: " & sPath
		
		CreateFolder = 0
		
		'Check and create the folder tree
		Do Until sFolder = sPath
			sFolder = Left(sPath, InStr(Len(sFolder) + 1, sPath, "\"))
			'UserLog.LogEvent INFORMATION, "[" & currentSub & "] Checking if folder exists: " & sFolder
			If Not oFso.FolderExists(sFolder) Then
				UserLog.LogEvent INFORMATION, "[" & currentSub & "] Creating folder: " & sFolder
				ofso.CreateFolder(sFolder)
				If Err.number <> 0 Then
					UserLog.LogEvent ERRORL, "[" & currentSub & "] Error creating: " & sFolder & ". The error was: " & Err.description
					CreateFolder = False
					Exit Function
				Else 
					CreateFolder = true
				End If
			End If
		Loop
	End Function

	function ReadFileLine(sPath, iLine)
		'**********************************************************************************
		' Purpose : Read a line from a text file
		' Input   : String : sPath : path to file
		'			integer : iLine : the line number to read
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:ReadFileLine"
		
		On Error Resume Next
		Err.clear
		
		If Helper.FileExists(sPath,False) Then
			aResult = Split(oFso.OpenTextFile(sPath,1).ReadAll,vbCRLF)
			
			'Just logging
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] Read Output: "
			For iCnt = 0 To UBound(aResult)
				UserLog.LogEvent INFORMATION, "[" & currentSub & "] -" & aResult(iCnt)
			Next

			If (iLine <= UBound(aResult)) Then
				ReadFileLine = aResult(iLine-1)
			End If
		End if		
		
		If Err.number <> 0 Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] Failed to Read File: " & sPath & ". The error was: " & Err.description & " (" & Err.number & ")"
		End If
	End Function
	
	Function DeleteFile(sPath)
		'**********************************************************************************
		' Purpose : Delete file
		' Input   : String : sPath : path to file
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:DeleteFile"
		
		'On Error Resume Next
		'Err.clear
		
		oFso.DeleteFile sPath, True
		
		If Err.number <> 0 Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] Failed to delete File: " & sPath & ". The error was: " & Err.description & " (" & Err.number & ")"
			DeleteFile = False
		Else
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] File: " & sPath & " was successfully deleted."
			DeleteFile = True 
		End If
	End Function
	
	Sub DeleteFolder(sPath)
		'**********************************************************************************
		' Purpose : Delete folder
		' Input   : String : sPath : path to folder
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:DeleteFolder"
		
		On Error Resume Next
		Err.clear
		
		oFso.DeleteFolder sPath, True
		
		If Err.number <> 0 Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] Failed to delete Folder: " & sPath & ". The error was: " & Err.description & " (" & Err.number & ")"
			DeleteFolder = False 
		Else
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] Folder: " & sPath & " was successfully deleted."
			DeleteFolder = True
		End If
	End Sub
	
	Function GetDayTimeDate()
		'**********************************************************************************
		' Purpose : Get Day,Date,Time
		' Input   : None
		' Return  : Current day of week + Date + Time
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helpers:GetDayTimeDate"
		
		GetDayTimeDate = WeekdayName(Weekday(Now, vbUseSystemDayOfWeek),,vbUseSystemDayOfWeek) & " " & Date & " "
		
		intSeconds = (Hour(Now) * 3600) + (Minute(Now) * 60) + Second(Now) 
        intMilliseconds = Timer() - intSeconds 
        'intMilliseconds = Fix(intMilliseconds * 100) 
        intMilliseconds = Fix(intMilliseconds * 10000) 
		
		If CInt(Hour(Now)) < 10 Then
			GetDayTimeDate = GetDayTimeDate & "0" & Hour(Now) & "."
		Else
			GetDayTimeDate = GetDayTimeDate & Hour(Now) & "."
		End If
		
		If CInt(Minute(Now)) < 10 Then
			GetDayTimeDate = GetDayTimeDate & "0" & Minute(Now) & "."
		Else
			GetDayTimeDate = GetDayTimeDate & Minute(Now) & "."
		End If
		
		If CInt(Second(Now)) < 10 Then
			GetDayTimeDate = GetDayTimeDate & "0" & Second(Now)
		Else
			GetDayTimeDate = GetDayTimeDate & Second(Now)
		End If
		
		If InStr(GetDayTimeDate, "/") Then
			GetDayTimeDate = Replace(GetDayTimeDate, "/", ".")
		End If
		If InStr(GetDayTimeDate, "\") Then
			GetDayTimeDate = Replace(GetDayTimeDate, "\", ".")
		End If
		
		GetDayTimeDate = GetDayTimeDate & "_" & intMilliseconds
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Creating Date " & GetDayTimeDate
	End Function
	
	Function Random(iValue)
		'**********************************************************************************
		' Purpose : Create a random integer between 0 and i
		' Input   : integer		: iValue	: The max value
		' Return  : integer		: Random number between 0 and Input (i)
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:Random"
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Creating a random integer between: 0-" & iValue
		
		'Generate seed
		Randomize
		iInt = Int(iValue * Rnd)
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Random integer is: " & iInt
		Random = iInt
	End Function
	
	Sub Terminate()
		'**********************************************************************************
		' Puropose : Terminate the script, moving log to error directory.
		'            Inform the user about the error.
		' Input    : None
		'**********************************************************************************
		Dim currentSub : currentSub = "Class Helper:Terminate"
		
		UserLog.LogEvent ERRORL, "[" & currentSub & "] ABORT."
		WScript.quit
	End Sub
	
	Function BreakString (str, str2, length, fillSpaces) 
		'********************************************************************************
		' Purpose	: Break up a string into multiline string having a given length
		' Input   	: string	: str		: the input string
		'			  string	: str2		: the input string to append at the end
		'			  integer	: length	: the desired final length
		' 			  bool	: fillSpaces: option to fill spaces
		'********************************************************************************
		Dim currentSub : currentSub = "Class Helper:BreakString"
		
		'fill with spaces
		ReDim aResult(0)
		
		iCnt = 0
		'process the main string
		While (Len(str) >length) 
			iWordBreak = InStrRev(str, " ", length)
			'force a break if no word break detected
			If (iWordBreak = 0) Then
				iWordBreak = length
			End If
			ReDim Preserve aResult(iCnt)
			aResult(iCnt) = Left(str, iWordBreak-1)
			str = Right(str,Len(str)-iWordBreak)
			iCnt = iCnt + 1
		Wend
		'check if the remaining string can accept the append of str2
		If (Len(str)+Len(str2)+1 >length) Then
			ReDim Preserve aResult(iCnt)
			aResult(iCnt) = str
			If (Len(Trim(str2))> 0) Then
				ReDim Preserve aResult(iCnt+1)
				If fillSpaces Then
					aResult(iCnt+1) = Space(length - Len(str2)) & str2
				Else
					aResult(iCnt+1) = str2
				End If
			End If
		Else
			ReDim Preserve aResult(iCnt)
			If fillSpaces Then
				aResult(iCnt) = str & Space(length - Len(str)- Len(str2)) & str2
			Else
				If (Len(str2) > 0) Then
					aResult(iCnt) = str & Space(1) & str2
				Else	
					aResult(iCnt) = str
				End If
			End If
		End If
		BreakString = aResult
	End Function
	
	Function BuildString (str, length)
		'********************************************************************************
		' Purpose : fill up a string with space to a given length
		' Input   : string	: str		: the input string
		'			integer : length	: the desired final length
		'********************************************************************************
		Dim currentSub : currentSub = "Class Helper:BuildString"
		
		'fill with spaces
		iCurrentLenth = Len(str)
		str = str & Space(length - iCurrentLenth)
		
		buildString = str
	End Function
	
	Function KeepOnly (sPath, iCount, iType)
		'**********************************************************************************
		' Purpose : Delete obsolete folders based upon creation date.
		'           n newest files are kept.
		' Input   : String (s) Path to root of folders
		'           Integer : iCount : Number of folders to keep
		'			Integer	: iType : 0 Files
		'							  1 Folders
		'							  2 Junctions
		'**********************************************************************************
		'On Error Resume Next
		On Error GoTo 0
		
		Dim currentSub : currentSub = "Class Helper:KeepOnly"
		
		Set oFldr = oFso.GetFolder(sPath)
		Helper.EchoErr Err, currentSub
		
		If Err.number <> 0 Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] An Error occured connecting to: " & sPath 
			KeepOnly = vbNull
			Exit function
		Else
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] Connected to: " & sPath
		End If
		
		'Switch Folder or file Mode
		Select Case (iType)
			Case 0
				sType = "File"
				Set cCollection = oFldr.Files
				Helper.EchoErr Err, currentSub
			Case 1
				sType = "Folder"
				Set cCollection = oFldr.SubFolders
				Helper.EchoErr Err, currentSub
			Case  2
				sType = "Junction"
				Set cCollection = oFldr.SubFolders
				Helper.EchoErr Err, currentSub
		End Select
		
		If Err.number <> 0 Then
			UserLog.LogEvent ERRORL, "[" & currentSub & "] An Error occured while listing " & sType & " in folder: " & sPath & ". Old " & sType & " must be deleted manually."
			KeepOnly = vbNull 
			Exit function
		Else
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] Succsessfully listed " & sType & " in folder: " & sPath
		End If
		
		'Check if number of files greater than i
		If cCollection.Count <= iCount Then
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] Number of " & sType & " in root is: " & cCollection.Count & ". Number is less or equal to: " & iCount & " wich is the number of " & sType & " to keep."
			KeepOnly = vbNull 
			Exit function
		End If
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Number of " & sType & " in root is: " & cCollection.Count & ". Number is greater: " & iCount & " wich is the number of " & sType & " to keep."
		
		'ReDim array to number of folders
		ReDim aDate(cCollection.Count -1)
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Creating array with fixed size: " & cCollection.Count -1 & ". Adding date of " & sType & " to array"
		
		iCnt = 0
		
		'Add date creation of all objects to array
		For Each oObject In cCollection
			UserLog.LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & oObject.DateCreated & " (Path: " & oObject.Path & ")"
			aDate(iCnt) = Array(oObject.DateCreated,oObject)
			iCnt = iCnt + 1
		Next
		
		'Sorting array by date, ascent sort order (The famous bubble sort)
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Sorting dates in array, ascent sort order"
		For n = 0 To UBound(aDate) -1 
			For m = n+1 To UBound(aDate) 
				If aDate(m)(0) < aDate(n)(0) Then
					temp = aDate(m)
					aDate(m) = aDate(n)
					aDate(n) = temp
				End If
			Next
		Next
		
		'Just logging
'		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Array sorted, result in ascent order: "
'		For iCnt = 0 To UBound(aDate)
'			UserLog.LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & aDate(iCnt)(0) & " (Path: " & aDate(iCnt)(1).Path & ")"
'		Next
		
		'Cut array, keep (i) rows with date of folders wich is going to be deleted
		ReDim Preserve aDate(UBound(aDate) - iCount)
		ReDim aDeleted (UBound(aDate))
'		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Array aDate cut, remaing rows are: "
		
		'Even more logging (those debuggers ;))
'		For iCnt = 0 To UBound(aDate)
'			UserLog.LogEvent INFORMATION, "[" & currentSub & "] aDate(" & iCnt & ") = " & aDate(iCnt)(0) & " (Path: " & aDate(iCnt)(1).Path & ")"
'		Next
		
		UserLog.LogEvent INFORMATION, "[" & currentSub & "] Start deleting " & sType & "..."
		For iCnt2 = 0 To UBound(aDate)
			aDeleted (iCnt2)	= aDate(iCnt2)(1).Name
			Select Case (sType)
				Case "File"
					Helper.DeleteFile aDate(iCnt2)(1).Path
				Case "Folder"
					Helper.DeleteFolder aDate(iCnt2)(1).Path
				Case "Junction"
					Junction.DeleteJunction aDate(iCnt2)(1).ParentFolder.Path , aDate(iCnt2)(1).Name
			End Select
		Next
		
		Helper.EchoErr Err, currentSub
		
		If Err.Number = 0 Then
			UserLog.LogEvent SUCCESS, "[" & currentSub & "] Deleted " & UBound(aDate)+1 & " obsolete " & sType & " Entries." 
		End If
		
		KeepOnly = aDeleted
	End function
End Class

Public Sub Class_Event(sSource, sEventType,ByVal bResult,ByVal values)
	'********************************************************************************
	' Purpose : Fire an event received from the child objects
	'********************************************************************************
	Dim currentSub : currentSub = "Main:Class_Event"
	
	If IsArray(values) Then
		values = Join(values,"|")
	ElseIf (values = vbNullString) Then
		values = "Null"
	End If 
	
	If (TypeName(DEBUG_CLASS_EVENTS) <> "Nothing") Then
		If (DEBUG_CLASS_EVENTS = True) Then
			WScript.Echo "EVENT FIRED [" & currentSub & "] source: " & sSource & " ; EventType:" & sEventType & " ; Result:" & bResult & " ; values:" & values
		End If
	End If 
End Sub


Sub Import (sFileName)
	'**********************************************************************************
	' Purpose : Import external file
	'**********************************************************************************
	Dim oFso, oFile
	Dim sScriptDir, sFilePath, sStreamData
	
	Set oFso		= CreateObject("Scripting.FileSystemObject")
	sScriptDir = oFso.GetParentFolderName(WScript.ScriptFullName)   
	sFilePath = oFso.BuildPath(sScriptDir,sFileName)   
	Const ForReading = 1   
	
	Set oFile = oFso.OpenTextFile(sFilePath,ForReading)   
	sStreamData = oFile.ReadAll()   
	ExecuteGlobal sStreamData   
End Sub